package no.usit.fridaws;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>
 * SjekkServlet blir brukt for å sjekke status for applikasjonen.
 * </p>
 * 
 * <p>
 * Kan gi svar på om
 * <ul>
 * <li>applikasjonen er oppe og svarer</li>
 * <li>db-oppkoblinger kan hentes ut for de konfigurerte dbene</li>
 * <li>redirigering til feide er riktig satt opp
 * </ul>
 * 
 * Ut over dette blir for tiden ingenting sjekket.
 * </p>
 * 
 * @author leivhe
 * 
 * @see https://hjelp.uio.no/Ticket/Display.html?id=464409
 * @see http://www.usit.uio.no/sapp/gap/w3d/arkiv/feilkoder.html
 * 
 */
public class SjekkServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;
    private static final Logger log = LoggerFactory.getLogger(SjekkServlet.class.toString());
    private static final String JDBC_CONTEXT = "java:/jdbc";


    /**
     * Returnerer status på servicem
     * 
     */
    @Override
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {

        PrintWriter writer = response.getWriter();
        StringBuilder sb = new StringBuilder();

        try {
            Context context = new InitialContext();
            sjekkDBOppkobling(context, "FDS", sb);
        } catch (NamingException e) {
            throw new RuntimeException(e);
        }

        if (sb.length() == 0) {
            writer.println("00 ok");
        } else {
            writer.print(sb.toString());
        }

    }


    /**
     * Finner og tester alle konfigurerte db-oppkoblinger.
     * 
     * @param feilbuffer
     *            info om evt. feil skrives til dette bufferet
     */
    /*
    protected void sjekkDBOppkoblinger(StringBuilder feilbuffer) {
        log.debug("sjekkDBOppkoblinger()");

        NamingEnumeration<NameClassPair> resources = null;
        Context context = null;
        try {
            context = new InitialContext();
            resources = context.list(JDBC_CONTEXT);
        } catch (NamingException e) {
            throw new RuntimeException(e);
        }

        while (resources.hasMoreElements()) {
            NameClassPair ncPair = resources.nextElement();
            sjekkDBOppkobling(context, ncPair.getName(), feilbuffer);
        }
    }
    */

    /**
     * Finn db-konneksjon-pool i JNDI og hent ut en konneksjon.
     * 
     * @param context
     *            Navne-kontekst der navneoppslag blir gjort.
     * @param dbKonneksjonPoolnavn
     *            kortnavn for dbkonn-poolen hvorfra konneksjon hentes
     * @param feilbuffer
     *            feil blir rapportert her
     * 
     */
    protected void sjekkDBOppkobling(Context context, String dbKonneksjonPoolnavn, StringBuilder feilbuffer) {
        Connection conn = null;
        String fullName = JDBC_CONTEXT + "/" + dbKonneksjonPoolnavn;
        try {
            DataSource ds = (DataSource) context.lookup(fullName);
            conn = ds.getConnection();
        } catch (NamingException e) {
            log.warn("doLookup() svelger NamingException for " + fullName);
            String prefiks = "70 warn jdbc-navngiving ";
            feilbuffer.append(prefiks);
            feilbuffer.append("NamingException for " + fullName + ": " + e.getMessage() + "\n");
            appendCausedBys(feilbuffer, prefiks, e);

        } catch (SQLException e) {
            log.warn("doLookup() svelger SQLException for " + fullName);
            String prefiks = "10 warn Database (" + fullName + ") ";
            feilbuffer.append(prefiks);
            feilbuffer.append("SQLException for " + fullName + ": " + e.getMessage() + "\n");
            appendCausedBys(feilbuffer, prefiks, e);

        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                }
            }
        }
    }


    private void appendCausedBys(StringBuilder sb, String prefiks, Throwable e) {
        log.debug("appendCausedBys()");
        while (e.getCause() != null) {
            e = e.getCause();
            sb.append(prefiks + "Caused by " + e.getMessage() + "\n");
        }
    }
}
