package no.usit.fridaws;

import java.util.ArrayList;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Response;

import no.usit.fridaws.exception.NotFoundException;
import no.usit.fridaws.exception.TooManyException;

import org.jboss.resteasy.annotations.cache.Cache;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * VarbeidSted RPC-tjeneste.
 */
@Path("/hentVarbeidSted")
public class VarbeidSted {

    private static final Logger logger = LoggerFactory.getLogger(VarbeidSted.class);


    /**
     * Returnerer Varbeidene til et sted.
     * 
     * @param instnr
     *            stedets institusjonsnr.
     * @param avdnr
     *            stedets avdelingsnr.
     * @param undavdnr
     *            stedets underavdelingsnr.
     * @param gruppenr
     *            stedets gruppenr.
     * @param format
     *            "xml" eller "json" for å angi output format
     * @param fra
     *            hent kun varbeider utgitt fra og med dette årstallet
     * @param til
     *            hent kun varbeider utgitt til og med dette årstallet
     * @param utplukk
     *            "alle", "nyeste" eller "viktigste" arbeider
     * @param maksantall
     *            begrenser svaret til dette antallet varbeider
     * @param hovedkategori
     *            begrens til kun denne hovedkategorien
     * @param underkategori
     *            begrens til kun denne underkategorien
     * @param sortering
     *            sorteringsangivelse, se bruker-doc.
     * @return en liste med varbeider
     */
    @GET
    @Cache(maxAge = 0)
    public Response hentVarbeidSted(@QueryParam("instnr") Long instnr,
            @QueryParam("avdnr") Long avdnr,
            @QueryParam("undavdnr") Long undavdnr,
            @QueryParam("gruppenr") Long gruppenr,
            @QueryParam("format") String format,
            @QueryParam("fra") Integer fraAar,
            @QueryParam("til") Integer tilAar,
            @QueryParam("utplukk") String utplukk,
            @QueryParam("maksantall") Long maksAntall,
            @QueryParam("hovedkategori") String hovedKategori,
            @QueryParam("underkategori") String underKategori,
            @QueryParam("sortering") String sortering) {
        if (format == null) {
            format = "xml";
        } else {
            format = format.toLowerCase();
        }

        if (utplukk == null) {
            utplukk = "alle";
        } else {
            utplukk = utplukk.toLowerCase();
        }

        if (hovedKategori != null) {
            hovedKategori = hovedKategori.toUpperCase();
        }
        if (underKategori != null) {
            underKategori = underKategori.toUpperCase();
        }

        if ((instnr == null) || (!format.equals("xml") && !format.equals("json"))
            || (fraAar != null && tilAar != null && fraAar > tilAar)
            || (!utplukk.equals("nyeste") && !utplukk.equals("viktigste") && !utplukk.equals("alle"))
            || (maksAntall != null && utplukk.equals("alle"))
            || (maksAntall != null && maksAntall < 1)) {
            return Response.status(400).build();
        }
        if (maksAntall == null || maksAntall > 1000) {
            maksAntall = 1001L;
        }

        String output = null;

        ArrayList<Object> params = new ArrayList<Object>();
        String sql, sFields;

        sFields = "vx.tekst_xml";
        if (utplukk.equals("viktigste")) {
            sql = "SELECT %SFIELDS% " + "FROM   varbeid_xml vx, "
                  + "       presentasjon_enhet pe, "
                  + "       presentasjon_varbeid pv, "
                  + "       vitenskapeligarbeid v "
                  + "WHERE  pe.presentasjonslopenr = pv.presentasjonslopenr "
                  + "  AND  pv.varbeidlopenr = v.varbeidlopenr "
                  + "  AND  vx.varbeidlopenr = v.varbeidlopenr "
                  + "  AND  pe.institusjonsnr = ? ";
        } else {
            sql = "SELECT %SFIELDS% " + "FROM   varbeid_xml vx, "
                  + "       vitenskapeligarbeid v "
                  + "WHERE  vx.varbeidlopenr = v.varbeidlopenr "
                  + "  AND  vx.varbeidlopenr IN "
                  + "		(SELECT DISTINCT varbeidlopenr FROM varbeid_person_sted vps"
                  + "        WHERE vps.institusjonsnr = ? ";
            params.add(instnr);
        }

        if (avdnr != null) {
            sql += " and avdnr = ? ";
            params.add(avdnr);
        }
        if (undavdnr != null) {
            sql += " and undavdnr = ? ";
            params.add(undavdnr);
        }
        if (gruppenr != null) {
            sql += " and gruppenr = ? ";
            params.add(gruppenr);
        }

        sql += ")";

        if (fraAar != null) {
            sql += " and v.arstall >= ? ";
            params.add(fraAar);
        }
        if (tilAar != null) {
            sql += " AND v.arstall <= ? ";
            params.add(tilAar);
        }

        if (hovedKategori != null) {
            sql += " AND v.varbeidhovedkatkode = ? ";
            params.add(hovedKategori);
        }
        if (underKategori != null) {
            sql += " AND v.varbeidunderkatkode = ? ";
            params.add(underKategori);
        }

        if (utplukk.equals("nyeste")) {
            sql += " ORDER BY v.arstall DESC ";
        }
        if (sortering != null) {
            String sortpred = Filter.lagVarbeidSortering(sortering);
            if (sortpred == null) {
                return Response.status(400).build();
            }
            if (utplukk.equals("nyeste")) {
                sql += ", " + sortpred + " ";
            } else {
                sql += " ORDER BY " + sortpred + " ";
            }
        }
        // til slutt, begrens antall: (dette maa gjoeres sist)
        if (maksAntall != null && maksAntall != 0) {
            sql = sql.replace("%SFIELDS%", sFields);
            sFields = "tekst_xml";
            sql = "SELECT %SFIELDS% FROM (" + sql + ") WHERE ROWNUM <= ?";
            params.add(maksAntall);
        }

        try {
            ResponseStreamer stream = new ResponseStreamer(
                                                           "http://frida.usit.uio.no/" + "eksport/forskningsresultater/fridaEksport.xsd",
                                                           "forskningsresultat", sql, sFields, params, format);
            return Xml.genererXML(format, stream);
        } catch (TooManyException e) {
            return Response.status(413).build();
        } catch (NotFoundException e) {
            return Response.status(404).build();
        } catch (Exception e) {
            return Response.status(500).build();
        }

    }

}
