package no.usit.fridaws;

import java.util.ArrayList;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Response;

import no.usit.fridaws.exception.NotFoundException;
import no.usit.fridaws.exception.TooManyException;

import org.apache.log4j.Logger;
import org.jboss.resteasy.annotations.cache.Cache;

/**
 * VarbeiderPerson RPC-tjeneste.
 */
@Path("/hentVarbeiderPerson")
public class VarbeiderPerson {

    // private static final Logger logger =
    // LoggerFactory.getLogger(VarbeiderPerson.class);

    private static final Logger logger = Logger.getLogger(VarbeiderPerson.class.toString());


    // logger = LoggerFactory .getLogger(VarbeiderPerson.class);

    /**
     * Returnerer Varbeidene til en person basert paa gitte kriterier.
     * 
     * @param navn
     *            personens brukernavn
     * @param eierkode
     *            stedet brukernavnet hører hjemme
     * @param lopenr
     *            personens unike loepenummer i databasen
     * @param format
     *            "xml" eller "json" for å angi output format
     * @param fra
     *            hent kun varbeider utgitt fra og med dette årstallet
     * @param til
     *            hent kun varbeider utgitt til og med dette årstallet
     * @param utplukk
     *            "alle", "nyeste" eller "viktigste" arbeider
     * @param maksantall
     *            begrenser svaret til dette antallet varbeider
     * @param hovedkategori
     *            begrens til kun denne hovedkategorien
     * @param underkategori
     *            begrens til kun denne underkategorien
     * @param sortering
     *            sorteringsangivelse, se bruker-doc.
     * @return en liste med varbeider
     */
    @GET
    @Cache(maxAge = 0)
    public Response hentVarbeiderPerson(@QueryParam("eierkode") String eierKode, @QueryParam("navn") String brukerNavn,
            @QueryParam("lopenr") Long lopenr, @QueryParam("format") String format, @QueryParam("fra") Integer fraAar,
            @QueryParam("til") Integer tilAar, @QueryParam("utplukk") String utplukk,
            @QueryParam("maksantall") Long maksAntall, @QueryParam("hovedkategori") String hovedKategori,
            @QueryParam("underkategori") String underKategori, @QueryParam("sortering") String sortering) {
        if (format == null) {
            format = "xml";
        } else {
            format = format.toLowerCase();
        }

        if (utplukk == null) {
            utplukk = "alle";
        } else {
            utplukk = utplukk.toLowerCase();
        }

        if (hovedKategori != null) {
            hovedKategori = hovedKategori.toUpperCase();
        }
        if (underKategori != null) {
            underKategori = underKategori.toUpperCase();
        }

        if (!InputValidering.okPerson(brukerNavn, eierKode, lopenr) || !(format.equals("xml") ^ format.equals("json"))
                || (fraAar != null && tilAar != null && fraAar > tilAar)
                || (!utplukk.equals("nyeste") && !utplukk.equals("viktigste") && !utplukk.equals("alle"))
                || (maksAntall != null && utplukk.equals("alle")) || (maksAntall != null && maksAntall < 1)) {
            return Response.status(400).build();
        }

        if (brukerNavn != null) {
            lopenr = Database.getPersonLopeNr(brukerNavn, eierKode);
        }
        if (lopenr == -1) {
            return Response.status(500).build();
        }

        if (lopenr == null) {
            return Response.status(404).build();
        }

        ArrayList<Object> params = new ArrayList<Object>();
        String sql, sFields;

        sFields = "vx.tekst_xml";

        if (utplukk.equals("viktigste")) {
            sql = "SELECT %SFIELDS% " + "FROM   varbeid_xml vx, " + "       vitenskapeligarbeid v, "
                    + "       presentasjon_varbeid pv, " + "       person p "
                    + "WHERE  p.presentasjonslopenr = pv.presentasjonslopenr "
                    + "  AND  pv.varbeidlopenr = v.varbeidlopenr " + "  AND  vx.varbeidlopenr = v.varbeidlopenr "
                    + "  AND  p.personlopenr = ? ";
        } else {
            sql = "SELECT %SFIELDS% " + "FROM   varbeid_xml vx, " + "       vitenskapeligarbeid v, "
                    + "       varbeid_person vp " + "WHERE  vx.varbeidlopenr = v.varbeidlopenr "
                    + "  AND  vp.varbeidlopenr = v.varbeidlopenr " + "  AND  vp.personlopenr = ? ";
        }
        params.add(lopenr);

        if (fraAar != null) {
            sql += " AND arstall >= ? ";
            params.add(fraAar);
        }
        if (tilAar != null) {
            sql += " AND arstall <= ? ";
            params.add(tilAar);
        }

        if (hovedKategori != null) {
            sql += " AND varbeidhovedkatkode = ? ";
            params.add(hovedKategori);
        }
        if (underKategori != null) {
            sql += " AND varbeidunderkatkode = ? ";
            params.add(underKategori);
        }
        if (utplukk.equals("nyeste")) {
            sql += " ORDER BY arstall desc ";
        }
        if (sortering != null) {
            String sortpred = Filter.lagVarbeidSortering(sortering);
            if (sortpred == null) {
                return Response.status(400).build();
            }
            if (utplukk.equals("nyeste")) {
                sql += ", " + sortpred + " ";
            } else {
                sql += " ORDER BY " + sortpred + " ";
            }
        }

        // til slutt, begrens antall: (dette maa gjoeres sist)
        if (maksAntall != null && maksAntall != 0) {
            sql = sql.replace("%SFIELDS%", sFields);
            sFields = "tekst_xml";
            sql = "SELECT %SFIELDS% FROM (" + sql + ") WHERE ROWNUM <= ?";
            // System.out.println("#### ny SQL: " + sql);
            params.add(maksAntall);
        }

        try {
            ResponseStreamer stream = new ResponseStreamer("http://frida.usit.uio.no/" + "xmlSchema/xsd/frida.xsd",
                    "forskningsresultat", sql, sFields, params, format);
            return Xml.genererXML(format, stream);
        } catch (TooManyException e) {
            return Response.status(413).build();
        } catch (NotFoundException e) {
            return Response.status(404).build();
        } catch (Exception e) {
            return Response.status(500).build();
        }

    }

}
